@@ -1,6 +1,6 @@
 #! /bin/sh
 :
-#ident	"@(#)smail/conf/lib:RELEASE-3_2_0_97:mkdefs.sh,v 1.51 1997/04/30 16:13:18 woods Exp"
+#ident	"@(#)smail/conf/lib:RELEASE-3_2_0_98:mkdefs.sh,v 1.52 1997/09/04 22:10:06 woods Exp"
 
 #    Copyright (C) 1987, 1988 Ronald S. Karr and Landon Curt Noll
 #    Copyright (C) 1992  Ronald S. Karr
@@ -25,7 +25,7 @@
 # from the EDITME file should override.
 
 # NOTE: The complete VARS string is too long for A/UX.  Split it into
-#	two parts which are concatenated.
+#	multiple parts which are concatenated.
 VARS="\
  ALIGNED_TYPE\
  ALIASES_FILE\
@@ -46,6 +46,8 @@ VARS="\
  FORCE_PATHS_TYPE\
  FORCE_SMTP_FILE\
  FORCE_SMTP_TYPE\
+ REWRITE_FILE\
+ REWRITE_TYPE\
  GATEWAY_NAMES\
  HOSTNAMES\
  LIB_DIR\
@@ -54,12 +56,12 @@ VARS="\
  LOCAL_MAIL_ADDRS\
  LOCAL_MAIL_COMMAND\
  LOCAL_MAIL_FILE\
+"
+VARS="$VARS\
  LOCKING_PROTOCOL\
  LOCK_BY_NAME\
  LOG_DIR\
  MAILBOX_DIR\
-"
-VARS="$VARS\
  MAN1\
  MAN1_EXT\
  MAN5\
@@ -84,11 +86,11 @@ VARS="$VARS\
  SMTP_RECEIVE_COMMAND_TIMEOUT\
  SMTP_RECEIVE_MESSAGE_TIMEOUT\
  SPOOL_DIRS\
+"
+VARS="$VARS\
  STRIP\
  TMP_DIR\
  UNCOMPRESS\
-"
-VARS="$VARS\
  UNSHAR_MAP_DIR\
  USE_ASCII\
  USE_SYMLINKS\
@@ -137,6 +139,7 @@ NOEDVARS="\
  PATH_FORCE_PATHS_FILE\
  PATH_FORCE_SMTP_FILE\
  PATH_PATHS_FILE\
+ PATH_REWRITE_FILE\
  UTIL_PATH\
 "
 
@@ -527,6 +530,24 @@ yp | aliasyp)
 	;;
 esac
 
+case "$REWRITE_TYPE" in
+
+yp | aliasyp)
+	PATH_REWRITE_FILE="$REWRITE_FILE"
+	;;
+
+*)
+	case "$REWRITE_FILE" in
+	/*  )
+		PATH_REWRITE_FILE="$REWRITE_FILE"
+		;;
+	*   )
+		PATH_REWRITE_FILE="$LIB_DIR/$REWRITE_FILE"
+		;;
+	esac
+	;;
+esac
+
 # If a SMAIL_NAME is specified, add it to the OTHER_SMAIL_NAMES.
 # Otherwise, create one that points to the main smail binary.
 case "$SMAIL_NAME" in
@@ -738,6 +759,9 @@ FORCE_PATHS_PROTO=$FORCE_PATHS_TYPE
 FORCE_SMTP_FILE=$FORCE_SMTP_FILE
 PATH_FORCE_SMTP_FILE=$PATH_FORCE_SMTP_FILE
 FORCE_SMTP_PROTO=$FORCE_SMTP_TYPE
+REWRITE_FILE=$REWRITE_FILE
+PATH_REWRITE_FILE=$PATH_REWRITE_FILE
+REWRITE_PROTO=$REWRITE_TYPE
 EOF
 
 # Handle string variables which should revert to defaults if undefined
@@ -893,6 +917,7 @@ s|X_LOGFILE_X|$LOGFILE|g
 s|X_PATHS_FILE_X|$PATHS_FILE|g
 s|X_FORCE_PATHS_FILE_X|$FORCE_PATHS_FILE|g
 s|X_FORCE_SMTP_FILE_X|$FORCE_SMTP_FILE|g
+s|X_REWRITE_FILE_X|$REWRITE_FILE|g
 s|X_PANICLOG_X|$PANICLOG|g
 s|X_OLD_LOGDIR_X|$OLD_LOGDIR|g
 s|X_MAIN_SPOOL_DIR_X|$MAIN_SPOOL_DIR|g
