@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_97:addr.c,v 1.19 1997/06/16 18:42:50 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_98:addr.c,v 1.21 1997/09/24 00:50:25 woods Exp"
  */
 
 /*
@@ -284,6 +284,7 @@ parse_address(address, target, remainder, flagp)
     int switch_flag;
 
     DEBUG1(DBG_ADDR_HI, "parse_address called: address=%s\n", address);
+
     /*
      * make sure we have an address
      */
@@ -482,7 +483,7 @@ parse_address(address, target, remainder, flagp)
 	DEBUG1(DBG_ADDR_HI, "parse_address: LOCAL: remainder=%s\n", *remainder);
 #else
 	DEBUG2(DBG_ADDR_HI, "parse_address: LOCAL: target=%s, remainder=%s\n",
-	       *target, *remainder);
+	       *target ? *target : "(no-domain)", *remainder);
 #endif
     }
     return LOCAL;
