@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_97:bindlib.c,v 1.39 1997/08/08 18:46:08 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_98:bindlib.c,v 1.44 1997/10/16 02:38:01 woods Exp"
  */
 
 /*
@@ -463,6 +463,8 @@ bind_compute_domain ()
 
 #define LOCALHOST_MX_CHECK	"localhost MX check"
 
+static struct bindlib_private localhost_bindattr = BIND_TEMPLATE_ATTRIBUTES;
+
 int
 bind_check_if_local_mxs(cur)
     struct addr *cur;
@@ -483,7 +485,7 @@ bind_check_if_local_mxs(cur)
      */
     if ((_res.options & RES_INIT)  == 0) {
 	if (res_init() == EOF) {
-	    DEBUG(DBG_DRIVER_LO, "ERROR: res_init() failed - defering addresses\n");
+	    DEBUG(DBG_DRIVER_LO, "ERROR: res_init() failed - deferring addresses\n");
 	    cur->error = res_init_error(LOCALHOST_MX_CHECK);
 	    return FILE_AGAIN;
 	}
@@ -509,7 +511,7 @@ bind_check_if_local_mxs(cur)
 #endif
     strip_target = strip_dots(cur->target); /* Omit needless dots! */
 
-    mxdat = mx_work_area_constructor((struct bindlib_private *) NULL,
+    mxdat = mx_work_area_constructor(&localhost_bindattr,
 				     &cur->error, LOCALHOST_MX_CHECK, 0L,
 				     &found_server, &no_server);
 
@@ -586,6 +588,8 @@ bind_addr(raw_target, flags, priv, what, rt_info, error_p)
     long save_res_options;              /* like _res.options in <resolv.h> */
     int ret;
 
+    DEBUG2(DBG_DRIVER_MID, "bind_addr(%s) called from %s.\n", raw_target, what);
+
 #ifdef RES_INIT
     /*  This initialises the resolver.  If the resolver is so old that RES_INIT
      *  isn't defined then the routines will initialise themselves - that is
@@ -722,7 +726,7 @@ bind_addr_work(raw_target, flags, priv, what, rt_info, error_p)
          */
 
         if (flags & BIND_DOMAIN_REQUIRED && strchr(target, '.') == NULL) {
-	    DEBUG1(DBG_ROUTE_MID, "target '%s' is not a domain.\n", target);
+	    DEBUG1(DBG_DRIVER_MID, "target '%s' is not a domain.\n", target);
             return DB_NOMATCH;
         }
 
@@ -740,7 +744,7 @@ bind_addr_work(raw_target, flags, priv, what, rt_info, error_p)
             if (priv->required) {
                 if (match_full_or_end_domain(priv->required, mxdat->full_target) == NULL) {
                     mx_work_area_destructor(mxdat, FALSE);
-		    DEBUG1(DBG_ROUTE_HI, "target '%s' is not in one of the required domains.\n", target);
+		    DEBUG1(DBG_DRIVER_HI, "target '%s' is not in one of the required domains.\n", target);
                     return DB_NOMATCH;
                 }
             }
@@ -899,8 +903,8 @@ static int route_by_greybook(mxdat)
                mxdat->priv->uk_greybook_hosts_file);
         result = open_database(mxdat->priv->uk_greybook_hosts_file,
 			       mxdat->priv->uk_greybook_hosts_proto,
-			       (int) mxdat->priv->uk_greybook_host_retries,
-			       (int) mxdat->priv->uk_greybook_host_interval,
+			       mxdat->priv->uk_greybook_host_retries,
+			       mxdat->priv->uk_greybook_host_interval,
 			       (struct stat *)NULL, 
 			       &db_ptr,
 			       &error_text);
@@ -979,25 +983,27 @@ static int full_mx_lookup (target, mxdat, allow_mangling, priv)
     }
 
     if (mxdat->mx_hints == NULL) {
-	DEBUG1(DBG_ROUTE_MID, "target '%s' needs null MX.\n", target);
+	DEBUG1(DBG_DRIVER_MID, "target '%s' needs null MX.\n", target);
         result = handle_null_mxs(mxdat, priv);
         if (result != DB_SUCCEED) {
             return result;
         }
-        if (mxdat->priv->fallback_gateway) 
+        if (mxdat->priv->fallback_gateway) {
             add_mx_hint(&(mxdat->mx_hints), BIND_FALLBACK_PREFERENCE, 
                         mxdat->priv->fallback_gateway, 1);
+	}
     } else {
-        if (mxdat->priv->fallback_gateway) 
+        if (mxdat->priv->fallback_gateway) {
             add_mx_hint(&(mxdat->mx_hints), BIND_FALLBACK_PREFERENCE, 
                         mxdat->priv->fallback_gateway, 1);
+	}
         /* Can only filter MX records if there are some! */
         result = filter_mx_hints(mxdat);
         if (result != DB_SUCCEED) {
             return result;
         }
 	if (mxdat->mx_hints == NULL) {
-	    DEBUG1(DBG_ROUTE_MID, "target '%s' had no MXs after filtering.\n", target);
+	    DEBUG1(DBG_DRIVER_MID, "target '%s' had no MXs after filtering.\n", target);
 	    mxdat->flags &= ~BIND_MX_ONLY; /* no other way to add ourselves! */
 
 	    return handle_null_mxs(mxdat, priv);
@@ -1010,7 +1016,7 @@ static int full_mx_lookup (target, mxdat, allow_mangling, priv)
     }
 
     if (mxdat->mx_hints == NULL && !(mxdat->flags & BIND_LOCAL_MX_OKAY)) {
-	DEBUG1(DBG_ROUTE_MID, "target '%s' has no MX hints.\n", target);
+	DEBUG1(DBG_DRIVER_MID, "target '%s' has no MX hints.\n", target);
         return DB_NOMATCH;              /* Not sure if you really can get here! */
     } else {
         return DB_SUCCEED;
@@ -1068,7 +1074,7 @@ static int find_mx_records (target, mxdat, allow_mangling)
     if (result == DB_SUCCEED) {
         mxdat->matched_target = COPY_STRING(cur_target);
     } else {
-	DEBUG2(DBG_ROUTE_MID, "find_mx_records: target '%s' not matched, returning %d.\n", target, result);
+	DEBUG2(DBG_DRIVER_MID, "find_mx_records: target '%s' not matched, returning %d.\n", target, result);
     }
     return result;
 }
@@ -1168,6 +1174,7 @@ static int decode_cnames(mxdat)
              * actually an alias for some other domain name. The query
              * should be repeated with the canonical domain name.
              */
+	    DEBUG1(DBG_DRIVER_MID, "decode_cnames() found CNAME pointing at %s.\n", nambuf);
             result =  find_mx_records(nambuf, mxdat, FALSE);
             if (result != DB_SUCCEED) {
                 /* Cleanup code - deallocate storage */
@@ -1196,10 +1203,10 @@ static int handle_null_mxs(mxdat, priv)
 
     if (mxdat->flags & BIND_MX_ONLY) {
         /* We need MX records.  We have none - hence failed! */
-	DEBUG1(DBG_ROUTE_LO, "handle_null_mxs: no faking for %s with BIND_MX_ONLY!.\n", mxdat->matched_target);
+	DEBUG1(DBG_DRIVER_LO, "handle_null_mxs: no faking for %s with BIND_MX_ONLY!.\n", mxdat->matched_target);
         return DB_NEGMATCH;
     } else if (match_full_or_end_domain(priv->mx_domains, mxdat->matched_target) != NULL) {
-	DEBUG1(DBG_ROUTE_LO, "handle_null_mxs: mx_domains says no faking for %s!.\n", mxdat->matched_target);
+	DEBUG1(DBG_DRIVER_LO, "handle_null_mxs: mx_domains says no faking for %s!.\n", mxdat->matched_target);
         return DB_NEGMATCH;
     } else {
         /*
@@ -1208,19 +1215,30 @@ static int handle_null_mxs(mxdat, priv)
          * the query will be empty.  This is a special case.  If the
          * list is empty, mailers should treat it as if it contained
          * one RR, an MX RR with a preference value of 0, and a host
-         * name of REMOTE.      (I.e., REMOTE is its only MX).  In
+         * name of REMOTE.  (I.e., REMOTE is its only MX).  In
          * addition, the mailer should do no further processing on
          * the list, but should attempt to deliver the message to
-         * REMOTE.      The idea here is that if a domain fails to
+         * REMOTE.  The idea here is that if a domain fails to
          * advertise any information about a particular name we will
          * give it the benefit of the doubt and attempt delivery.
          */
 
 	/* Check if local host because if it is we don't need address hints! */
+	/*
+	 * XXX WARNING: things like wild-card MXs will break the islocalhost()
+	 * check!!!!
+	 */
 	if (islocalhost(mxdat->matched_target) && (mxdat->flags & BIND_LOCAL_MX_OKAY)) {
-	    DEBUG1(DBG_ROUTE_HI, "handle_null_mxs: islocalhost(%s).\n", mxdat->matched_target);
+	    DEBUG1(DBG_DRIVER_HI, "handle_null_mxs: islocalhost(%s).\n", mxdat->matched_target);
 	    return DB_SUCCEED;
 	}
+	/*
+	 * XXX it would be nice if we could re-add the lowest precedence MX
+	 * that had been filtered previously since this would be known to have
+	 * a valid hint (assuming the DNS was correct).  Of course if there
+	 * were no MXs filtered then we still have to fake up an MX for the
+	 * target domain and fill in a hint for it.
+	 */
         add_mx_hint(&(mxdat->mx_hints), 0,
                     mxdat->matched_target,
                     TRUE);
@@ -1247,7 +1265,8 @@ static int handle_null_mxs(mxdat, priv)
                 }
             }
         } else {
-	    DEBUG2(DBG_ROUTE_LO, "handle_null_mxs: no address hints for %s [%d].\n", mxdat->matched_target, result);
+	    DEBUG2(DBG_DRIVER_LO, "handle_null_mxs: no address hints for %s [%d].\n", mxdat->matched_target, result);
+	    /* XXX it's possible this is a config error and should go to paniclog */
 	}
 
         return result;                 /* A touch optomistic! */
@@ -1327,7 +1346,7 @@ static int filter_mx_hints(mxdat)
            (match_full_or_end_domain(mxdat->priv->uk_suffix, mxdat->full_target))) {
             if ((mxdat->priv->uk_max_precedence) &&
                 (mx_hint->preference > mxdat->priv->uk_max_precedence)) {
-                DEBUG2(DBG_ROUTE_HI, "Removed MX hint %s preference %d [UK transport]\n", 
+                DEBUG2(DBG_DRIVER_HI, "Removed MX hint %s preference %d [UK transport]\n", 
                        name, precedence);
                 free_mx_hint(mx_a);
                 mxdat->UK_MX_rejected = TRUE;
@@ -1336,7 +1355,7 @@ static int filter_mx_hints(mxdat)
       
             if ((mxdat->priv->uk_ignore_gateways != NULL) &&
                 (is_string_in_list(name, mxdat->priv->uk_ignore_gateways))) {
-                DEBUG2(DBG_ROUTE_HI, "Removed MX hint %s preference %d [UK gateways]\n", 
+                DEBUG2(DBG_DRIVER_HI, "Removed MX hint %s preference %d [UK gateways]\n", 
                        name, precedence);
                 free_mx_hint(mx_a);
                 mxdat->UK_MX_rejected = TRUE;
@@ -1350,7 +1369,7 @@ static int filter_mx_hints(mxdat)
              * RRs with a preference value greater than or equal to that
              * of LOCAL's must be discarded.
              */
-            DEBUG3(DBG_ROUTE_HI, "Removed MX hint %s preference %d [Local precedence %d]\n", 
+            DEBUG3(DBG_DRIVER_HI, "Removed MX hint %s preference %d [Local precedence %d]\n", 
                    name, precedence, mxdat->local_precedence);
             free_mx_hint(mx_a);
             continue;
@@ -1364,7 +1383,7 @@ static int filter_mx_hints(mxdat)
         if (mxdat->local_precedence >= 0) {
             /* This means we rejected because the local host is the prefered MX */
             if (mxdat->flags & BIND_LOCAL_MX_OKAY) {
-		DEBUG(DBG_ROUTE_HI, "filter_mx_hints(): returning DB_SUCCEED because BIND_LOCAL_MX_OKAY.\n");
+		DEBUG(DBG_DRIVER_HI, "filter_mx_hints(): returning DB_SUCCEED because BIND_LOCAL_MX_OKAY.\n");
                 return DB_SUCCEED;
             } else {
                 *(mxdat->error_p) = matched_local_host(mxdat->what, mxdat->matched_target);
@@ -1458,7 +1477,7 @@ static int get_addr_hints(mxdat)
     if (mxdat->mx_hints)
         return DB_SUCCEED;
     else {
-	DEBUG1(DBG_ROUTE_HI, "get_addr_hints: %s.\n", dns_timeout ? "dns timeout" : "no hints");
+	DEBUG1(DBG_DRIVER_HI, "get_addr_hints: %s.\n", dns_timeout ? "dns timeout" : "no hints");
         if (dns_timeout)
             return DB_AGAIN;
         else
@@ -2215,7 +2234,7 @@ add_mx_hint(addr, precedence, name, implicit)
 {
     struct transport_hints * new_hint;
 
-    DEBUG3(DBG_ROUTE_MID, "add_mx_hint: %s MX %d hint for %s.\n",
+    DEBUG3(DBG_DRIVER_MID, "add_mx_hint: %s MX %d hint for %s.\n",
 	   implicit ? "implicit" : "non-implicit", precedence, name)
     new_hint = new_mx_hint(precedence, COPY_STRING(name), implicit);
 #define mxhint(hint) ((struct mx_transport_hint *)((hint)->private))
