@@ -1,4 +1,4 @@
-/* @(#)src/bindlib.h bindlib.h,v 1.7 1997/04/21 17:48:22 woods Exp */
+/* @(#)src/bindlib.h bindlib.h,v 1.8 1997/10/16 02:51:10 woods Exp */
 
 /*
  *    Copyright (C) 1992  Ronald S. Karr
@@ -50,8 +50,8 @@ struct bindlib_private {
     char *uk_greybook_hosts_list;       /* UK: list of hosts I can gb to */
     char *uk_greybook_hosts_file;       /* UK: file of hosts I can gb to */
     char *uk_greybook_hosts_proto;      /* UK: proto for above */
-    char *uk_greybook_host_retries;     /* UK: retries for above */
-    char *uk_greybook_host_interval;    /* UK: interval for above */
+    int uk_greybook_host_retries;	/* UK: retries for above */
+    int uk_greybook_host_interval;	/* UK: interval for above */
     char *uk_suffix;                    /* UK: Suffix for UK addresses */
     int  rewrite_headers;               /* Amount of header rewriting to do */
     int  cname_limit;                   /* Depth of CNAMES allowed */
@@ -94,9 +94,9 @@ struct bindlib_private {
           OFFSET(STRUCT, BL_PRIV_MEM.uk_greybook_hosts_file) },         \
     { "uk_greybook_hosts_proto", t_string, NULL, NULL,                  \
           OFFSET(STRUCT, BL_PRIV_MEM.uk_greybook_hosts_proto) },        \
-    { "uk_greybook_host_retries", t_string, NULL, NULL,                 \
+    { "uk_greybook_host_retries", t_int, NULL, NULL,			\
           OFFSET(STRUCT, BL_PRIV_MEM.uk_greybook_host_retries) },       \
-    { "uk_greybook_host_interval", t_string, NULL, NULL,                \
+    { "uk_greybook_host_interval", t_int, NULL, NULL,			\
           OFFSET(STRUCT, BL_PRIV_MEM.uk_greybook_host_interval) },      \
     { "uk_suffix", t_string, NULL, NULL,                                \
           OFFSET(STRUCT, BL_PRIV_MEM.uk_suffix) },                      \
@@ -109,25 +109,27 @@ struct bindlib_private {
 
 #define BIND_TEMPLATE_FLAGS 0
 
+/* XXX Note that retries should be 2 ifndef HAVE_RENAME */
 #define BIND_TEMPLATE_ATTRIBUTES \
-   { NULL,                              /* char *match_domains */              \
-     NULL,                              /* char *ignore_domains */              \
-     NULL,                              /* char *required */                    \
-     NULL,                              /* char *widen_domains */               \
-     NULL,                              /* char *mx_domains */                  \
-     NULL,                              /* char *gateways */                    \
-     NULL,                              /* char *uk_ignore_gateways */          \
-     NULL,                              /* char *uk_greybook_transport */       \
-     0,                                 /* int  uk_max_precedence */            \
-     NULL,                              /* char *uk_greybook_relay */           \
-     NULL,                              /* char *uk_greybook_hosts_list */      \
-     NULL,                              /* char *uk_greybook_hosts_file */      \
-     NULL,                              /* char *uk_greybook_hosts_proto */     \
-     NULL,                              /* char *uk_greybook_host_retries */    \
-     NULL,                              /* char *uk_greybook_host_interval */   \
-     ".uk",                             /* char *uk_suffix */                   \
-     0,                                 /* int  rewrite_headers */              \
-     1,					/* int  cname_limit */                  \
-     NULL                               /* char *fallback_gateway */            \
+   { NULL,                              /* char *match_domains */		\
+     NULL,                              /* char *ignore_domains */		\
+     NULL,                              /* char *required */			\
+     NULL,                              /* char *widen_domains */		\
+     NULL,                              /* char *mx_domains */			\
+     NULL,                              /* char *gateways */			\
+     NULL,                              /* char *uk_ignore_gateways */		\
+     NULL,                              /* char *uk_greybook_transport */	\
+     0,                                 /* int  uk_max_precedence */		\
+     NULL,                              /* char *uk_greybook_relay */		\
+     NULL,                              /* char *uk_greybook_hosts_list */	\
+     NULL,                              /* char *uk_greybook_hosts_file */	\
+     NULL,                              /* char *uk_greybook_hosts_proto */	\
+     0,					/* char *uk_greybook_host_retries */	\
+     0,					/* char *uk_greybook_host_interval */	\
+     ".uk",                             /* char *uk_suffix */			\
+     0,                                 /* int  rewrite_headers */		\
+     1,					/* int  cname_limit */			\
+     NULL                               /* char *fallback_gateway */		\
    }
+
 #endif /* not BINDLIB_H */
