@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_97:config.h,v 1.55 1997/06/24 02:14:11 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_98:config.h,v 1.56 1997/09/04 22:10:22 woods Exp"
  */
 
 /*
@@ -1868,3 +1868,51 @@ ${if and{{origin:local} {def:owner}}\
 #ifndef	FORCE_SMTP_TRANSPORT
 # define FORCE_SMTP_TRANSPORT	"smtp"		/* deliver directly with smtp */
 #endif
+
+
+/*
+ * Default rewrite router setup
+ *
+ * If you wish to have a rewrite router by default, define REWRITE_FILE
+ * to be the name of a sorted rewrite database, and REWRITE_PROTO to be
+ * the database access protocol used for lookups.  See the file lookup.c
+ * for a complete list of these protocols.  If this file does not begin
+ * with `/', it will be referenced relative to the smail_lib_dir
+ * directory.  A current list of protocols is:
+ *
+ * lsearch - perform linear searches on raw path files.  This is slow
+ *	     unless the path file is small.
+ * bsearch - perform binary searches on sorted path files with one
+ *	     path per line.
+ * dbm	   - use the Berkeley dbm(3X) or ndbm(3X) libraries to search
+ *	     for paths.
+ * yp	   - use Sun's YP service to search for paths.  This requires
+ *	     the existence of the YP library routines.
+ *
+ * #define REWRITE_FILE	"/usr/smail/lib/rewrite" (* alternate suggestion *)
+ */
+#if	!defined(REWRITE_FILE) && !defined(NO_REWRITE_FILE)
+# define REWRITE_FILE	"rewrite"	/* Suggestion */
+#endif
+/* NOTE: this is derrived from REWRITE_TYPE in conf/EDITME by conf/lib/mkdefs.sh */
+#ifndef	REWRITE_PROTO
+# define REWRITE_PROTO	"bsearch"	/* Suggestion */
+#endif
+
+/*
+ * The rewrite file is always considered optional, so that if
+ * the file does not exist, it is assumed to be empty.
+ */
+#if	!defined(REWRITE_OPTIONAL) && !defined(NO_REWRITE_OPTIONAL)
+# define REWRITE_OPTIONAL		/*  */
+#endif
+
+/*
+ * In some cases, failure to open a paths database can be considered
+ * a temporary failure which can be recovered from by retrying the open
+ * at a later time.  One example would be use of YP where the server
+ * host may be down.  To enable this behavior, define the name below.
+ */
+#if	!defined(REWRITE_TRYAGAIN) && !defined(NO_REWRITE_TRYAGAIN)
+/* # define REWRITE_TRYAGAIN		(* you probably don't want this! *) */
+#endif
