@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_97:default.c,v 1.38 1997/06/25 04:45:43 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_98:default.c,v 1.41 1997/09/24 00:45:41 woods Exp"
  */
 
 /*
@@ -754,6 +754,63 @@ static struct router reroute_router = {
 #endif	/* EXPERIMENTAL_REROUTER */
 
 
+#if defined(REWRITE_FILE)
+/*
+ * REWRITE ROUTER:
+ *
+ * The name of the rewrite file and the access method type is
+ * defined in config.h, if a rewrite database is configured.
+ */
+
+static struct pathalias_private rewrite_router_priv = {
+    REWRITE_FILE,			/* file attribute */
+    REWRITE_PROTO,			/* file access method */
+    NULL,				/* default domain names */
+    NULL,				/* no required domain names */
+#ifdef	HAVE_RENAME
+    /*
+     * if we have an atomic rename call, retries should not be required
+     * since the file should always exist, even when being rebuilt
+     *
+     * if the file is not found, the pathalias director is ignored.
+     */
+    0,					/* no retries */
+#else	/* HAVE_RENAME */
+    /*
+     * without an atomic rename call, there will be short periods
+     * of time when the file does not exist, after it is has been
+     * rebuilt between when it is removed and the new one is put
+     * into place.  (Assuming one doesn't just edit it in place!)
+     */
+    2,					/* two retries on open */
+#endif	/* HAVE_RENAME */
+    2,					/* two second intervals for retries */
+    NULL,				/* internal - open database */
+    NULL,				/* internal - error text from open */
+};
+
+static struct router rewrite_router = {
+    "rewrite",				/* routing through rewriter */
+    "rewrite",				/* use the rewrite driver */
+    SUCC,				/* point to next router */
+    USE_ALWAYS |
+#ifdef REWRITE_OPTIONAL
+	PA_OPTIONAL |			/* the rewrite file is optional */
+#endif
+#ifdef REWRITE_TRYAGAIN
+	PA_TRYAGAIN |			/* defer address on open failure */
+#endif
+	PA_REOPEN,			/* always reopen database to search */
+    METHOD_TABLE,			/* use method table to find tport */
+    NULL,				/* no default transport */
+    (char *) &rewrite_router_priv,	/* rewrite private configuration */
+};
+#undef SUCC
+#define SUCC (&rewrite_router)	/* setup for forward link */
+
+#endif /* REWRITE_FILE */
+
+
 #if defined(FORCE_PATHS_FILE)
 /*
  * FORCE_PATH ROUTER:
@@ -950,7 +1007,7 @@ static struct transport pipe_transport = {
     /* put Recieved:, Return-Path: and From_ line, will do local delivery */
     PUT_RECEIVED | PUT_RETURNPATH | LOCAL_TPORT | LOCAL_XFORM | PUT_FROM |
 	UNIX_FROM_HACK |
-	PIPE_AS_USER | PIPE_PARENT_ENV | PIPE_IGNORE_STATUS | PIPE_IGNORE_WRERRS,
+	PIPE_AS_USER | PIPE_PARENT_ENV | PIPE_LOG_OUTPUT | PIPE_STATUS_2SENDER,
     1, 1, 0,				/* 1 addr, 1 host, size not useful */
     NULL, NULL, NULL,			/* hdrremove, hdrinsert, hdrappend */
     NULL,				/* retry directory */
@@ -1363,7 +1420,7 @@ static struct transport idemand_uusmtp_transport = {
 #endif
 
 #ifndef SMTP_LONG_TIMEOUT
-#define SMTP_LONG_TIMEOUT (5*60*60)	/* long timeout, 2 hours */
+#define SMTP_LONG_TIMEOUT  (5*60*60)	/* long timeout, 5 hours */
 #endif
 
 #ifndef SMTP_SERVICE_ENTRY
