@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_97:direct.c,v 1.18 1997/06/11 04:00:59 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_98:direct.c,v 1.19 1997/09/24 00:51:52 woods Exp"
  */
 
 /*
@@ -618,22 +618,35 @@ finish_direct_other_form(dp, addr)
 	     * the next director.  E.g., foo aliased
 	     * to foo.
 	     */
-	    DEBUG3(DBG_DIRECT_LO,
-		   "    directed %s --> %s ... match localhost, pass %s to next director\n",
-		   addr->parent->remainder, addr->work_addr, temp_remainder);
+	    DEBUG4(DBG_DIRECT_LO,
+		   "    directed %s@%s --> %s ... match localhost, pass %s to next director\n",
+		   addr->parent->remainder,
+		   addr->parent->target ? addr->parent->target : "(no-parent-target)",
+		   addr->work_addr,
+		   temp_remainder);
 	    (void) strcpy(addr->work_addr, temp_remainder);
 	    addr->remainder = addr->work_addr;
 	    xfree(temp_work_addr);
+
 	    return SEND_TO_NEXT;
-	} else {
-	    /* This prevents accidental matching of remainder */
-	    addr->flags |= ADDR_DONTHASH;
 	}
+	/* This prevents accidental matching of remainder */
+	addr->flags |= ADDR_DONTHASH;
+	DEBUG6(DBG_DIRECT_HI,
+	       "    NOT passing %s@%s to next director as %s%s%s%s\n",
+	       addr->parent->remainder,
+	       addr->parent->target ? addr->parent->target : "(no-parent-target)",
+	       addr->work_addr,
+	       (islocalhost(temp_target) ? "" :  ", not a localhost"),
+	       (EQIC(temp_remainder, addr->parent->remainder) ? "" : ", not same remainder"),
+	       (EQIC(temp_target, addr->parent->target) ? "" : ", not same target"));
     }
 
     /* we have a new address to reparse */
-    DEBUG2(DBG_DIRECT_LO, "    directed %s --> %s\n",
-	   addr->parent->remainder, addr->in_addr);
+    DEBUG3(DBG_DIRECT_LO, "    directed %s@%s --> %s\n",
+	   addr->parent->remainder,
+	   addr->parent->target ? addr->parent->target : "(no-parent-target)",
+	   addr->in_addr);
 
     return ADDRESS_OKAY;
 }
