@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_97:extern.h,v 1.57 1997/06/25 04:47:12 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_98:extern.h,v 1.58 1997/10/16 02:52:25 woods Exp"
  */
 
 /*
@@ -486,7 +486,7 @@ extern int read_spool();
 extern void log_spool_errors();
 extern int new_grade();
 extern void defer_message();
-extern long message_date();
+extern time_t message_date();
 extern long spool_max_free_space();
 
 /* external functions defined in string.c */
