@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_97:modes.c,v 1.66 1997/07/01 02:09:02 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_98:modes.c,v 1.70 1997/10/16 02:52:29 woods Exp"
  */
 
 /*
@@ -38,8 +38,10 @@
 #include <signal.h>
 #include <errno.h>
 #include <sys/param.h>
-#ifndef ISC_SOCKET_TIME_BUG
+#if (defined(POSIX_OS) || !defined(UNIX_BSD)) && !defined(ISC_SOCKET_TIME_BUG)
 # include <time.h>
+#else
+# include <sys/time.h>
 #endif
 #if defined(UNIX_BSD) && !defined(POSIX_OS)
 # include <sys/ioctl.h>
@@ -428,7 +430,7 @@ test_addresses()
 	build_host_strings();
     }
 
-    while (stdin_is_a_tty && fputs("> ", stdout)) {
+    while (stdin_is_a_tty && (fputs("> ", stdout) != EOF)) {
 	struct addr *cur = alloc_addr(); /* XXX memory leak.... */
 	struct addr *done;
 	struct addr *retry;
@@ -793,7 +795,7 @@ deliver_mail()
     }
 
     if (defer != NULL) {
-	long message_age = (long) time((time_t *) NULL) - (long) message_date();
+	long message_age = (long) (time((time_t *) NULL) - message_date());
 	if (message_age > resolve_timeout) {
 	    /*
 	     * This message has been waiting for delivery longer than the
@@ -873,6 +875,7 @@ deliver_mail()
 
 
 static void daemon_sighup();
+static void daemon_sigterm();
 static void daemon_sigalrm();
 #ifdef SIGCHLD
 static void daemon_sigchld();
@@ -998,24 +1001,29 @@ daemon_mode()
     (void) signal(SIGCHLD, daemon_sigchld);
 
     /* assure that max connects is a sane value */
-    if (smtp_accept_max < 0 || smtp_accept_max > 4095) {
+    if (smtp_accept_max < 0) {
 	smtp_accept_max = 0;
     }
-    smtp_accept_slots = smtp_accept_max;
-    /* assure that max connects until queue only is a sane value */
-    if (smtp_accept_queue < 0 || smtp_accept_queue > 4095) {
+    if (smtp_accept_max > 4095) {
+	smtp_accept_max = 4095;
+    }
+    /* assure that max connects until only queueing is a sane value */
+    if (smtp_accept_queue < 0) {
 	smtp_accept_queue = 0;
     }
-    if (smtp_accept_queue > smtp_accept_slots)
-	smtp_accept_slots = smtp_accept_queue;
-
-    /* allocate space for listener pids (if we're keeping track) */
+    if (smtp_accept_queue > 4095) {
+	smtp_accept_queue = 4095;
+    }
+    /* assure that there are enough slots for all possible child pids */
+    smtp_accept_slots = (smtp_accept_queue > smtp_accept_max) ? smtp_accept_queue : smtp_accept_max;
+    /* allocate space for connected child pids (if we're keeping track) */
     if (smtp_pids == NULL && smtp_accept_slots > 0) {
 	int i;
 
 	smtp_pids = (int *) xmalloc((unsigned) (smtp_accept_slots * sizeof(int)));
-	for (i = 0; i < smtp_accept_slots; ++i)
+	for (i = 0; i < smtp_accept_slots; ++i) {
 	    smtp_pids[i] = 0;
+	}
     }
 #endif /* SIGCHLD */
 
@@ -1041,6 +1049,7 @@ daemon_mode()
     /* SIGHUP means re-exec */
     got_sighup = FALSE;
     (void) signal(SIGHUP, daemon_sighup);
+    (void) signal(SIGTERM, daemon_sigterm);
 
     /* set the alarm for wakeup time */
     got_sigalrm = FALSE;
@@ -1145,7 +1154,7 @@ daemon_mode()
 	    close_system_logs();
 	    open_system_logs();
 
-	    /* recache all of the driver info, to get any changes */
+	    /* re-cache all of the driver info, to get any changes */
 #ifdef SIGCHLD
 	    (void) signal(SIGCHLD, SIG_DFL);
 #endif
@@ -1201,23 +1210,34 @@ do_daemon_accept(ls, fd, from)
 	(void) close(fd);
 	return;
     }
-    pid = fork();
-    if (pid == 0) {
+    /* don't worry about retries here -- the remote should retry for us... */
+    switch (pid = fork()) {
+    case -1: {		/* fork() failed in the parent process */
+	int oerrno = errno;
+	FILE *f = fdopen(fd, "w");	/* XXX error check this? */
+
+	DEBUG3(DBG_MAIN_MID, "fork() failed for pid %d: rejecting SMTP connection #%d: %s\n",
+	       getpid(), smtp_accept_count + 1, strerror(oerrno));
+	write_log(LOG_SYS, "pid %d: fork() failed: %s", getpid(), strerror(oerrno));
+	(void) fprintf(f, "421 %s Connection refused -- system resources exausted\r\n",
+		       primary_name);
+	(void) fflush(f);
+	break;
+    }
+    case 0: {		/* in the child process */
 	FILE *in;			/* input channel */
 	FILE *out;			/* output channel */
 #ifdef HAVE_LIBWRAP
 	struct request_info req;
 #endif
 
-	/* in child process */
 	(void) close(ls);
 	/* setup the channels */
 	in = fdopen(fd, "r");		/* XXX error check this? */
 	out = fdopen(fd2, "w");		/* XXX error check this? */
 
-	/* no longer ignore kids */
 #ifdef SIGCHLD
-	(void) signal(SIGCHLD, SIG_DFL);
+	(void) signal(SIGCHLD, SIG_DFL); /* child may fork() in delivery */
 #endif
 
 #ifdef HAVE_LIBWRAP
@@ -1227,8 +1247,8 @@ do_daemon_accept(ls, fd, from)
 #endif
 	/*
 	 * if the number of outstanding child processes exceeds
-	 * smtp_accept_queue, then turn on queue_only, so that
-	 * mail will not be delivered immediately.
+	 * smtp_accept_queue, then turn on queue_only in the child,
+	 * so that mail will not be delivered immediately.
 	 */
 	if (smtp_accept_queue > 0 && smtp_accept_count >= smtp_accept_queue) {
 	    DEBUG1(DBG_MAIN_MID, "many connections, use queue_only #%d...\n",
@@ -1241,19 +1261,14 @@ do_daemon_accept(ls, fd, from)
 
 	/* done with that transaction */
 	exit(0);
-    }
-    /* in parent process */
-    if (pid < 0) {
-	FILE *f = fdopen(fd, "w");
 
-	(void) fprintf(f, "421 %s Connection refused: %s\r\n",
-		       primary_name, strerror(errno));
-	(void) fflush(f);
+	/* NOTREACHED */
     }
-    else {
+    default: {		/* in the parent process */
 #ifdef SIGCHLD
 	int i;
 
+	/* squirrel away child pid in next free slot & increment count */
 	for (i = 0; i < smtp_accept_slots; ++i) {
 	    if (smtp_pids[i] <= 0) {
 		smtp_pids[i] = pid;
@@ -1262,9 +1277,12 @@ do_daemon_accept(ls, fd, from)
 	    }
 	}
 #endif /* SIGCHLD */
-    }
+	break;
+    } }		/* end of switch() */
     (void) close(fd);
     (void) close(fd2);
+
+    return;
 }
 
 #else	/* not defined(HAVE_BSD_NETWORKING) */
@@ -1298,6 +1316,18 @@ daemon_sighup(sig)
     (void) signal(SIGHUP, daemon_sighup);
 }
 
+/*
+ * daemon_sighup - note that we received a SIGHUP signal
+ */
+/*ARGSUSED*/
+static void
+daemon_sigterm(sig)
+    int sig;
+{
+    write_log(LOG_SYS, "pid %d: SIGTERM received, shutting down daemon.", getpid());
+    exit(EX_OK);
+}
+
 #ifdef SIGCHLD
 /*
  * daemon_sigchld - reap dead kids
@@ -1322,7 +1352,7 @@ daemon_sigchld(sig)
 	reap_child(pid);
     }
 
-#ifndef UNIX_BSD
+#if !defined(UNIX_BSD) || (defined(sun) && defined(UNIX_SYS5))
     (void) signal(sig, daemon_sigchld);
 #endif
 }
@@ -1433,7 +1463,7 @@ noop_mode()
 	    close_system_logs();
 	    open_system_logs();
 
-	    /* recache all of the driver info, to get any changes */
+	    /* re-cache all of the driver info, to get any changes */
 #ifdef SIGCHLD
 	    (void) signal(SIGCHLD, SIG_DFL);
 #endif
@@ -1544,7 +1574,7 @@ bg_run_queue(ls)
 	(void) setpgrp(0, getpid());
 #endif	/* POSIX_OS */
 #ifdef SIGCHLD
-	/* in child process we care about dying kids */
+	/* in child process we want dying kids to be reaped right away */
 	(void) signal(SIGCHLD, SIG_DFL);
 #endif
 	(void) alarm(0);
@@ -1671,6 +1701,7 @@ verify_addresses()
 
     for (cur = deliver; cur; cur = cur->succ) {
 	if (cur->next_host) {
+	    DEBUG1(DBG_DRIVER_LO, "    transport is %s\n", cur->transport->name);
 	    printf("%s at %s ... deliverable\n",
 		   cur->next_addr, cur->next_host);
 	} else {
