@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_97:notify.c,v 1.26 1997/08/06 06:19:08 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_98:notify.c,v 1.29 1997/09/22 23:31:55 woods Exp"
  */
 
 /*
@@ -24,7 +24,6 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <pwd.h>
-#include <signal.h>
 #include <time.h>
 #include "defs.h"
 #ifdef HAVE_STDLIB_H
@@ -771,26 +770,18 @@ notify(defer, fail, sent_errors)
 	(void) fputs(".\n", f);		/* finish the message */
     }
 
-    /* if the postmaster is to receive mail, then send it to him (her?) */
+    /* if the postmaster is to receive mail, then send a copy */
     if (send_to_postmaster) {
-	int wrote_addr_error_banner = FALSE; /* only write banner once */
 
 	cc1 = cc2 = NULL;
 	if (return_to_sender)
 	    cc2 = sender;
-	notify_new_message(f, "postmaster", cc1, cc2,
-			   "sending to postmaster");
+	notify_new_message(f, "postmaster", cc1, cc2, "sending to postmaster");
 	send_log(f, TRUE, log_banner);	/* include X lines from log */
 	/* write messages in to_other destined to the postmaster */
+	(void) fputs(addr_error_banner, f);
 	for (cur = to_other; cur; cur = cur->succ) {
-	    if (cur->error->info & ERR_NPOSTMAST) {
-		if (!wrote_addr_error_banner) {
-		    (void) fputs(addr_error_banner, f);
-		    wrote_addr_error_banner = TRUE;
-		}
-		(void) fprintf(f, " %s ... %s\n",
-			       cur->in_addr, cur->error->message);
-	    }
+	    (void) fprintf(f, " %s ... %s\n", cur->in_addr, cur->error->message);
 	}
 	(void) fputs(text_banner, f);
 	write_notify_header(f);
@@ -799,9 +790,8 @@ notify(defer, fail, sent_errors)
 	(void) fputs(".\n", f);		/* finish the message */
     }
 
-    /* if the sender is to receive mail, arrange for that */
+    /* if the sender is to receive mail, then send a copy */
     if (return_to_sender) {
-	int wrote_addr_error_banner = FALSE; /* only write banner once */
 
 	cc1 = cc2 = NULL;
 	if (send_to_postmaster)
@@ -809,27 +799,19 @@ notify(defer, fail, sent_errors)
 	notify_new_message(f, sender, cc1, cc2, "returning to sender");
 	send_log(f, FALSE, log_banner);	/* don't include X lines from log */
 	/* write messages in to_other destined to the sender */
+	(void) fputs(addr_error_banner, f);
 	for (cur = to_other; cur; cur = cur->succ) {
-	    if (cur->error->info & ERR_NSENDER) {
-		if (!wrote_addr_error_banner) {
-		    (void) fputs(addr_error_banner, f);
-		    wrote_addr_error_banner = TRUE;
-		}
-		(void) fprintf(f, " %s ... %s\n",
-			       cur->in_addr, /* XXX this is never complete for multi-delivery msgs */
-			       cur->error->message);
-	    }
+	    (void) fprintf(f, " %s ... %s\n", cur->in_addr, cur->error->message);
 	}
 	(void) fputs(text_banner, f);
-
+	write_notify_header(f);
+	putc('\n', f);
 	/*
 	 * a message grade between a and z causes body not to be returned.
 	 * This is handy when notification of errors is desired, but
 	 * when the overhead of sending the entire message back is not
 	 * necessary.
 	 */
-	write_notify_header(f);
-	putc('\n', f);
 	if (islower(msg_grade)) {
 	    fputs("[low-priority message, body not included]\n", f);
 	} else {
