@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_97:queue.c,v 1.33 1997/06/26 02:15:21 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_98:queue.c,v 1.34 1997/08/20 23:35:26 woods Exp"
  */
 
 /*
@@ -1182,13 +1182,18 @@ log_incoming()
 
     if (sender_host) {
 	char *sender_really_different = NULL;
+	char *sender_host_not_literal = NULL;
 
 	if (sender_host_really) {
 	    if (strcmpic(sender_host, sender_host_really) != 0) {
 		sender_really_different = sender_host_really;
 	    }
 	}
-	host_string = xprintf("%s%s%s%s%s%s%s", sender_host,
+	if (!sender_host_addr || (strcmpic(sender_host, sender_host_addr) != 0)) {
+	    sender_host_not_literal = sender_host;
+	}
+	host_string = xprintf("%s%s%s%s%s%s%s",
+			      sender_host_not_literal ? sender_host_not_literal : "",
 			      sender_really_different ? "(" : "",
 			      sender_really_different ? sender_really_different : "",
 			      sender_really_different ? ")" : "",
