@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_97:smailconf.c,v 1.37 1997/06/11 05:03:31 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_98:smailconf.c,v 1.38 1997/10/16 02:56:37 woods Exp"
  */
 
 /*
@@ -342,7 +342,7 @@ quote_string_value(str)
     static struct str val;
     static int initialised = FALSE;
     char s;
-    char buf[4];
+    char buf[5];		/* must hold '\\', '2', '3', '4', '\0' */
 
     if (!initialised) {
 	STR_INIT(&val);
