@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_97:smtprecv.c,v 1.55 1997/08/19 06:13:21 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_98:smtprecv.c,v 1.65 1997/09/04 17:52:50 woods Exp"
  */
 
 /*
@@ -211,6 +211,10 @@ receive_smtp(in, out)
     struct sockaddr_in from_sa;
     int from_sa_len = sizeof(from_sa);
     struct hostent *shp = NULL;		/* result of gethostbyaddr() */
+
+# ifndef OBSOLETE_RESOLVER
+    extern int h_errno;			/* import h_errno; many systems don't define it in <netdb.h> */
+# endif
 #endif /* HAVE_BSD_NETWORKING */
 
     /* initialize state */
@@ -314,7 +318,11 @@ receive_smtp(in, out)
 	/* WARNING: must check shp contents before calling gethost*() again */
 	if (!(shp = gethostbyaddr((char *) &(from_sa.sin_addr),
 			      sizeof(from_sa.sin_addr), AF_INET))) {
+# ifndef OBSOLETE_RESOLVER
 	    DEBUG2(DBG_REMOTE_LO, "gethostbyaddr(%s) failed: %s.\n", sender_host_addr, hstrerror(h_errno));
+# else
+	    DEBUG1(DBG_REMOTE_LO, "gethostbyaddr(%s) failed.\n", sender_host_addr);
+# endif
 	    sender_host_really = NULL;
 	} else {
 	    sender_host_really = COPY_STRING(shp->h_name);
@@ -400,7 +408,11 @@ receive_smtp(in, out)
 		    /* we've been around once before and need to reset... */
 		    if (!(shp = gethostbyaddr((char *) &(from_sa.sin_addr),
 					      sizeof(from_sa.sin_addr), AF_INET))) {
+# ifndef OBSOLETE_RESOLVER
 			DEBUG2(DBG_REMOTE_LO, "gethostbyaddr(%s) failed: %s.\n", sender_host_addr, hstrerror(h_errno));
+# else
+			DEBUG1(DBG_REMOTE_LO, "gethostbyaddr(%s) failed.\n", sender_host_addr);
+# endif
 			sender_host_really = NULL;
 		    } else {
 			sender_host_really = COPY_STRING(shp->h_name);
@@ -413,22 +425,24 @@ receive_smtp(in, out)
 			    ident_sender ? ident_sender : "",
 			    ident_sender ? "@" : "",
 			    sender_host_really ? sender_host_really : "",
-			    sender_host_addr ? "[" : "",
+			    sender_host_addr ? " remote address [" : "",
 			    sender_host_addr ? sender_host_addr : "",
 			    sender_host_addr ? "]" : "",
 			    error ? ": " : "", error ? error : "");
 		    fflush(out);
 		}
-		if (!sender_host || error) { /* log even "hidden" errors */
-		    write_log(LOG_SYS, "remote %s: invalid operand: '%s': from %s%s%s%s%s%s%s%s%s%s%s.",
-			      ehlo_p ? "EHLO" : "HELO", orig_data,
+		if (!sender_host || error) { /* log even "hidden" errors as warnings */
+		    write_log(LOG_SYS, "remote %s: %s operand: '%s': from %s%s%s%s%s%s%s%s%s%s%s.",
+			      ehlo_p ? "EHLO" : "HELO",
+			      sender_host ? "questionable" : "invalid",
+			      orig_data,
 			      ident_sender ? ident_sender : "",
 			      ident_sender ? "@" : "",
 			      sender_host ? sender_host : "",
 			      sender_host_really ? "(" : "",
 			      sender_host_really ? sender_host_really : "",
 			      sender_host_really ? ")" : "",
-			      sender_host_addr ? "[" : "",
+			      sender_host_addr ? " source [" : "",
 			      sender_host_addr ? sender_host_addr : "",
 			      sender_host_addr ? "]" : "",
 			      error ? ": " : "",
@@ -439,19 +453,23 @@ receive_smtp(in, out)
 		    break;		/* broken DNS or forger */
 		}
 #else
-		DEBUG1(DBG_REMOTE_LO, "receive_smtp(): no networking support so not verifying sender host %s.\n", data);
+		DEBUG1(DBG_REMOTE_LO, "receive_smtp(): no BSD networking, so not verifying sender host %s.\n", data);
 #endif /* HAVE_BSD_NETWORKING */
 	    } else if (!sender_host && data[0] != '\0') {
 		sender_host = COPY_STRING(data);
 	    }
 	    if (out) {
+		/* XXX RFC 0821 says "The receiver-SMTP identifies itself to
+		 * the sender-SMTP in [...] the response to this command
+		 * [HELO]."
+		 */
 		fprintf(out, "250%s%s Hello %s (%s%s%s%s%s%s)%s\r\n",
 			ehlo_p ? "-" : " ",
 			primary_name, data,
 			ident_sender ? ident_sender : "",
 			ident_sender ? "@" : "",
-			sender_host_really ? sender_host_really : sender_host ? sender_host : "(UNKNOWN)",
-			sender_host_addr ? "[" : "",
+			sender_host_really ? sender_host_really : (sender_host ? sender_host : "(UNKNOWN)"),
+			sender_host_addr ? " from address [" : "",
 			sender_host_addr ? sender_host_addr : "",
 			sender_host_addr ? "]" : "",
 			ehlo_p ? ", here's what we support:" : ".");
@@ -657,6 +675,7 @@ receive_smtp(in, out)
 		fflush(out);
 	    }
 	    break;
+
 	  fail_mail_cmd:
 	    if (sender) {
 		xfree(sender);
@@ -1206,7 +1225,7 @@ verify_sender(pp_sender, raw_sender, out)
 	return 0;
     }
     if (*vaddr->work_addr && !EQ(vaddr->work_addr, "+") && form == LOCAL) {
-	fprintf(out, "501 '%s' sender address cannot be a local address.\r\n", vaddr->work_addr);
+	fprintf(out, "501 '%s' sender address cannot be a local or unqualified address (i.e. it must have a domain portion).\r\n", vaddr->work_addr);
 	fflush(out);
 	write_log(LOG_SYS, "remote MAIL FROM: '%s' is a local address; by %s%s%s%s%s%s.",
 		  vaddr->in_addr,
@@ -1297,7 +1316,7 @@ verify_sender(pp_sender, raw_sender, out)
 	case DB_AGAIN:			/* DNS lookup must be deferred */
 	case FILE_AGAIN:		/* lost contact with server */
 	case FILE_NOMATCH:		/* There is no server available */
-	    fprintf(out, "450 defer '%s', sender address target '%s' cannot be verified at this time: %s.\r\n",
+	    fprintf(out, "450 defer '%s', sender address target '%s' cannot be verified at this time: %s. (Try again later)\r\n",
 		    vaddr->in_addr, vaddr->target,
 		    (binderr->message && *binderr->message) ? binderr->message : "Unknown error");
 	    fflush(out);
@@ -1344,7 +1363,7 @@ verify_sender(pp_sender, raw_sender, out)
 	    return 0;
 
 	default:
-	    fprintf(out, "421 '%s' panic trying to verify address target '%s': %s.\r\n",
+	    fprintf(out, "421 '%s' impossible error when trying to verify address target '%s': %s. (Try again later)\r\n",
 		    vaddr->in_addr, vaddr->target,
 		    (binderr->message && *binderr->message) ? binderr->message : "Unknown error");
 	    fflush(out);
@@ -1384,7 +1403,7 @@ verify_addr(in_addr, out, rcpt_p)
     /* build the mungeable addr string */
     addr->work_addr = preparse_address(in_addr, &error);
     if (addr->work_addr == NULL) {
-	fprintf(out, "501 '%s' malformed address; %s.\r\n", in_addr, error);
+	fprintf(out, "501 '%s' malformed address: %s.\r\n", in_addr, error);
 	xfree(addr);
 	return 0;
     }
@@ -1407,10 +1426,26 @@ verify_addr(in_addr, out, rcpt_p)
  	return 1;
     } else if (defer) {
 	/* XXX hope that the other guy eventually gives up */
- 	fprintf(out, "450 defer '%s' <%s@%s> cannot verify at this time: (ERR_%03ld) %s.\r\n",
+ 	fprintf(out, "450 defer '%s' <%s@%s> cannot verify at this time: (ERR_%03ld) %s. (Try again later)\r\n",
 		in_addr, defer->work_addr,
 		(defer->target) ? defer->target : "(nodomain)",
 		defer->error->info & ERR_MASK, defer->error->message);
+	if (rcpt_p) {
+	    /* XXX this may be very noisy.... */
+	    write_log(LOG_SYS, "remote %s%s%s%s%s%s: '%s' <%s@%s> temporary failure returned to '%s' while verifying recipient: (ERR_%03ld) %s",
+		      ident_sender ? ident_sender : "",
+		      ident_sender ? "@" : "",
+		      sender_host,
+		      sender_host_addr ? "[" : "",
+		      sender_host_addr ? sender_host_addr : "<no-sender-addr>",
+		      sender_host_addr ? "]" : "",
+		      in_addr,
+		      defer->work_addr,
+		      (defer->target) ? defer->target : "(nodomain)",
+		      sender,
+		      defer->error->info & ERR_MASK,
+		      defer->error->message);
+	}
 	xfree(addr);
  	return 1;
     } else if (fail) {
@@ -1434,17 +1469,22 @@ verify_addr(in_addr, out, rcpt_p)
 	    error_code = 553;		/* use this to flag un-xlated errors */
 	    break;
 	}
- 	fprintf(out, "%d '%s' <%s@%s> not matched: %s.\r\n", error_code, in_addr,
+ 	fprintf(out, "%d '%s' <%s@%s> not matched: (ERR_%03ld) %s.\r\n", error_code, in_addr,
 		fail->work_addr, (fail->target) ? fail->target : "(nodomain)",
-		fail->error->message);
+		fail->error->info & ERR_MASK, fail->error->message);
 	if (rcpt_p) {
-	    write_log(LOG_SYS, "remote %s%s%s%s%s%s: '%s' <%s@%s> recipient not matched: %s",
-		      ident_sender ? ident_sender : "", ident_sender ? "@" : "",
+	    write_log(LOG_SYS, "remote %s%s%s%s%s%s: '%s' <%s@%s> recipient for sender '%s' not matched: (ERR_%03ld) %s",
+		      ident_sender ? ident_sender : "",
+		      ident_sender ? "@" : "",
 		      sender_host,
 		      sender_host_addr ? "[" : "",
 		      sender_host_addr ? sender_host_addr : "<no-sender-addr>",
 		      sender_host_addr ? "]" : "",
-		      in_addr, fail->work_addr, (fail->target) ? fail->target : "(nodomain)",
+		      in_addr,
+		      fail->work_addr,
+		      (fail->target) ? fail->target : "(nodomain)",
+		      sender,
+		      fail->error->info & ERR_MASK,
 		      fail->error->message);
 	}
 	xfree(addr);
@@ -1455,14 +1495,15 @@ verify_addr(in_addr, out, rcpt_p)
 	    addr->work_addr, (addr->target) ? addr->target : "(nodomain)");
     if (rcpt_p) {
 	/* XXX do we really need to log these? */
-	write_log(LOG_SYS, "remote %s%s%s%s%s%s: '%s' <%s@%s> recipient not matched",
+	write_log(LOG_SYS, "remote %s%s%s%s%s%s: '%s' <%s@%s> recipient for sender '%s' not matched by anything!",
 		  ident_sender ? ident_sender : "", ident_sender ? "@" : "",
 		  sender_host,
 		  sender_host_addr ? "[" : "",
 		  sender_host_addr ? sender_host_addr : "<no-sender-addr>",
 		  sender_host_addr ? "]" : "",
 		  in_addr, addr->work_addr,
-		  (addr->target) ? addr->target : "(nodomain)");
+		  (addr->target) ? addr->target : "(nodomain)",
+		  sender);
     }
     xfree(addr);
     return 0;
@@ -1473,18 +1514,8 @@ verify_addr(in_addr, out, rcpt_p)
 /*
  * verify_host - verify the sender hostname in the DNS
  *
- * If the hostname is an address literal then we first match it with the saddrp
- * and return NULL if it doesn't match.  If it does match we try looking up the
- * PTR RR for it, and if we find one we return a copy of that name, otherwise
- * we fail with NULL.
- *
- * [[Address Literals *should* be followed by helpful information....]]
- *
- * For all other hostnames we look up the A RR for the name supplied, the PTR
- * RR for the resulting A RR, verify that they match, then verify that the
- * sender_host_address also matches.
- *
- * Returns either sender_host_really or hostnm if verify succeeds.
+ * Returns hostnm if verify succeeds.
+ * (possibly with the text that followed a domain literal form appended).
  */
 static char *
 verify_host(hostnm, saddrp, shp, errorp)
@@ -1499,7 +1530,9 @@ verify_host(hostnm, saddrp, shp, errorp)
     int i;
     int host_not_broken = TRUE;
 
+# ifndef OBSOLETE_RESOLVER
     extern int h_errno;			/* import h_errno; many systems don't define it in <netdb.h> */
+# endif
 
     if (smtp_hello_broken_allow) {
 	host_not_broken = !match_ip(inet_ntoa(saddrp->sin_addr), smtp_hello_broken_allow);
@@ -1544,7 +1577,11 @@ verify_host(hostnm, saddrp, shp, errorp)
 	*p = ']';
 	if (shp && sender_host_really) { /* both from gethostbyaddr() in caller */
 	    if (!(hp = gethostbyname(sender_host_really))) {
+# ifndef OBSOLETE_RESOLVER
 		*errorp = (char *) hstrerror(h_errno);
+# else
+		*errorp = "obsolete resolver(3) -- no clues.";
+# endif
 		DEBUG2(DBG_REMOTE_LO, "verify_host(%s) gethostbyname() for verify_literal failed: %s.\n", sender_host_really, *errorp);
 		if (smtp_hello_verify_literal && host_not_broken) {
 		    return NULL;	/* broken DNS or mailer config; or forger */
@@ -1586,7 +1623,9 @@ verify_host(hostnm, saddrp, shp, errorp)
     if (*hostnm == '.') {
 	*errorp = "hostname must not start with '.'";
 	DEBUG1(DBG_REMOTE_LO, "hostname must not start with '.': %s", hostnm);
-	return NULL;
+	if (host_not_broken) {
+	    return NULL;
+	}
     }
     found = 0;
     for (p = hostnm; *p; p++) {
@@ -1595,13 +1634,17 @@ verify_host(hostnm, saddrp, shp, errorp)
 	} else if (!(isalnum(*p) || *p == '-')) {
             *errorp = "illegal character in hostname";
             DEBUG2(DBG_REMOTE_LO, "illegal char in hostname: %s, %c.\n", hostnm, *p);
-            return NULL;
+	    if (host_not_broken) {
+		return NULL;
+	    }
         }
     }
     if (!found) {
 	*errorp = "hostname must contain a '.'";
 	DEBUG1(DBG_REMOTE_LO, "hostname must contain a '.': %s", hostnm);
-	return NULL;
+	if (host_not_broken) {
+	    return NULL;
+	}
     }
     /* NOTE: must check shp contents before calling gethost*() again */
     if (shp) {				/* from gethostbyaddr() in caller */
@@ -1635,7 +1678,11 @@ verify_host(hostnm, saddrp, shp, errorp)
 	}
     }
     if (!(hp = gethostbyname(hostnm))) { /* NOTE: must have checked shp by now.... */
+# ifndef OBSOLETE_RESOLVER
 	*errorp = (char *) hstrerror(h_errno);
+# else
+	*errorp = "obsolete resolver(3) -- no clues.";
+# endif
 	DEBUG2(DBG_REMOTE_LO, "verify_host(%s) gethostbyname() failed: %s.\n", hostnm, *errorp);
 	/* XXX it might be nice to return a temporary error here, but that would mean a re-design */
 	if (smtp_hello_verify && host_not_broken) {
