@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_97:spool.c,v 1.20 1997/07/01 02:11:56 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_98:spool.c,v 1.22 1997/10/16 02:52:37 woods Exp"
  */
 
 /*
@@ -34,7 +34,6 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <errno.h>
-#include <signal.h>
 #include "defs.h"
 #ifdef HAVE_STDLIB_H
 # include <stdlib.h>
@@ -1631,10 +1630,10 @@ new_grade(grade)
  * base 62 number.  Return the date as the number of seconds since
  * the epoch.
  */
-long
+time_t
 message_date()
 {
-    long clck;
+    time_t clck;
     register int digit;
     register char *p;
 
