@@ -1,4 +1,8 @@
+#include <stdio.h>
+
 #include "varargs.h"
+#include "smail.h"
+#include "extern.h"
 
 /*
  * get_local_year stub for get_local_year in sysdep.c
