@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_97:sysdep.c,v 1.60 1997/08/04 05:51:44 woods Exp
+#ident	"@(#)smail/src:RELEASE-3_2_0_98:sysdep.c,v 1.64 1997/10/16 21:43:42 woods Exp
  */
 
 /*
@@ -29,7 +29,6 @@
 #include <ctype.h>
 #include <errno.h>
 #include <pwd.h>
-#include <signal.h>
 #include "defs.h"
 #include "smail.h"
 #include "child.h"
@@ -85,7 +84,7 @@
 # endif	/* UNIX_XENIX && !POSIX_OS */
 #endif	/* UNIX_BSD && !POSIX_OS */
 
-#if defined(POSIX_OS) || !defined(UNIX_BSD)
+#if (defined(POSIX_OS) || !defined(UNIX_BSD)) && !defined(ISC_SOCKET_TIME_BUG)
 # include <time.h>
 #else
 # include <sys/time.h>
@@ -350,29 +349,29 @@ static char *
 get_time_zone_name(ltm)
 struct tm *ltm;
 {
-#if	defined(USE_TM_TZNAME)
+#if defined(USE_TM_ZONE) || defined(sun)
     return ltm->tm_zone;
-#else	/* not USE_TM_TZNAME */
+#else
 # if defined(UNIX_SYS5) || defined(POSIX_OS) || defined(USE_TZNAME)
     extern char *tzname[2];
 
     return tzname[ltm->tm_isdst];
-# else	/* not UNIX_SYS5, not POSIX_OS, and not USE_TZNAME */
+# else
 #  if	(defined(UNIX_BSD) || defined(USE_GETTIMEOFDAY)) 
     struct timeval tv;
     struct timezone tz;
 
     (void) gettimeofday(&tv, &tz);
     return timezone(tz.tz_minuteswest, ltm->tm_isdst);
-#   else	/* (!UNIX_BSD || !USE_GETTIMEOFDAY) */
+#  else
     struct timeb timeb;
 
     /* try the V6/V7 system call for getting the time zone */
     (void) ftime(&timeb);
     return timezone(timeb.timezone, ltm->tm_isdst);
-#  endif	/* not UNIX_BSD */
-# endif	/* not UNIX_SYS5 */
-#endif	/* USE_TM_TZNAME */
+#  endif
+# endif
+#endif
 }
 
 /*
@@ -551,23 +550,23 @@ fopen_as_user(fn, type, uid, gid, mode)
 		    if (setgid(gid) != 0) {
 			DEBUG3(DBG_DRIVER_LO, "fopen_as_user(): create child setgid(%d): %s (uid was %d).\n",
 			       gid, strerror(errno), uid);
-			_exit(errno + 1);
+			exit(errno + 1);
 			/* NOTREACHED */
 		    }
 		}
 		if (setuid(uid) != 0) {
 		    DEBUG2(DBG_DRIVER_LO, "fopen_as_user(): create child setuid(%d): %s.\n",
 			   uid, strerror(errno));
-		    _exit(errno + 1);
+		    exit(errno + 1);
 		    /* NOTREACHED */
 		}
 		if ((cfd = open(fn, O_WRONLY | O_CREAT, mode)) < 0) {
 		    DEBUG2(DBG_DRIVER_LO, "fopen_as_user(): create open(%s): %s.\n", fn, strerror(errno));
-		    _exit(errno + 1);
+		    exit(errno + 1);
 		    /* NOTREACHED */
 		}
 		close(cfd);
-		_exit(0);
+		exit(0);
 		/* NOTREACHED */
 
 	    default:
@@ -579,16 +578,22 @@ fopen_as_user(fn, type, uid, gid, mode)
 			return NULL;
 		    }
 		}
-		if (WIFEXITED(status) && WEXITSTATUS(status)) {
-		    DEBUG1(WEXITSTATUS(status) - 1 ? DBG_DRIVER_LO : DBG_DRIVER_HI,
-			   "fopen_as_user(): create child exited with %d.\n",
-			   WEXITSTATUS(status) - 1);
-		    errno = WEXITSTATUS(status) - 1;
-		    return NULL;
-		}
-		if (WIFSIGNALED(status) || WIFSTOPPED(status)) {
+		if (WIFEXITED(status)) {
+		    if (WEXITSTATUS(status)) {
+			DEBUG1(WEXITSTATUS(status) - 1 ? DBG_DRIVER_LO : DBG_DRIVER_HI,
+			       "fopen_as_user(): create child exited with %d.\n",
+			       WEXITSTATUS(status) - 1);
+			errno = WEXITSTATUS(status) - 1;
+			return NULL;
+		    }
+		} else if (WIFSIGNALED(status)) {
 		    DEBUG2(DBG_DRIVER_LO, "fopen_as_user(): create child[%d] signalled %d.\n",
-			   pid, WTERMSIG(status));	/* XXX what if stopped???? */
+			   pid, WTERMSIG(status));
+		    errno = EINTR;
+		    return NULL;
+		} else if (WIFSTOPPED(status)) {
+		    DEBUG2(DBG_DRIVER_LO, "fopen_as_user(): create child[%d] stopped %d.\n",
+			   pid, WSTOPSIG(status));
 		    errno = EINTR;
 		    return NULL;
 		}
@@ -641,14 +646,14 @@ fopen_as_user(fn, type, uid, gid, mode)
 		if (setgid(gid) != 0) {
 		    DEBUG4(DBG_DRIVER_LO, "fopen_as_user(): test child setgid(%d): %s (uid=%d now, to become %d).\n",
 			   gid, strerror(errno), getuid(), uid);
-		    _exit(errno + 1);
+		    exit(errno + 1);
 		    /* NOTREACHED */
 		}
 	    }
 	    if (setuid(uid) != 0) {
 		DEBUG2(DBG_DRIVER_LO, "fopen_as_user(): test child setuid(%d): %s.\n",
 		       uid, strerror(errno));
-		_exit(errno + 1);
+		exit(errno + 1);
 		/* NOTREACHED */
 	    }
 	    if (type[0] == 'w') {
@@ -665,7 +670,7 @@ fopen_as_user(fn, type, uid, gid, mode)
 	    if (cfd < 0) {
 		DEBUG2(DBG_DRIVER_LO, "fopen_as_user(): test child open(%s): %s.\n",
 		       fn, strerror(errno));
-		_exit(errno + 1);
+		exit(errno + 1);
 		/* NOTREACHED */
 	    }
 	    if (fstat(cfd, &stb_user) < 0) {
@@ -674,16 +679,16 @@ fopen_as_user(fn, type, uid, gid, mode)
 		DEBUG2(DBG_DRIVER_LO, "fopen_as_user(): test child fstat(%s): %s.\n",
 		       fn, strerror(errno));
 		close(cfd);
-		_exit(oerrno + 1);
+		exit(oerrno + 1);
 	    }
 	    close(cfd);
 	    if ((stb_root.st_dev != stb_user.st_dev) ||
 		(stb_root.st_ino != stb_user.st_ino)) {
 		write_log(LOG_SYS|LOG_PANIC,	/* XXX child writing to log here!!! */
 			  "fopen_as_user: parent/child file mismatch!: %s.\n", fn);
-		_exit(EACCES + 1);		/* got a better idea? */
+		exit(EACCES + 1);		/* got a better idea? */
 	    }
-	    _exit(0);
+	    exit(0);
 	    /* NOTREACHED */
 
         default:
@@ -695,16 +700,22 @@ fopen_as_user(fn, type, uid, gid, mode)
 		    return NULL;
 		}
 	    }
-	    if (WIFEXITED(status) && WEXITSTATUS(status)) {
-		DEBUG1(WEXITSTATUS(status) - 1 ? DBG_DRIVER_LO : DBG_DRIVER_HI,
-		       "fopen_as_user(): test child exited with %d.\n",
-		       WEXITSTATUS(status) - 1);
-		errno = WEXITSTATUS(status) - 1;
-		return NULL;
-	    }
-	    if (WIFSIGNALED(status) || WIFSTOPPED(status)) {
+	    if (WIFEXITED(status)) {
+		if (WEXITSTATUS(status)) {
+		    DEBUG1(WEXITSTATUS(status) - 1 ? DBG_DRIVER_LO : DBG_DRIVER_HI,
+			   "fopen_as_user(): test child exited with %d.\n",
+			   WEXITSTATUS(status) - 1);
+		    errno = WEXITSTATUS(status) - 1;
+		    return NULL;
+		}
+	    } else if (WIFSIGNALED(status)) {
 		DEBUG2(DBG_DRIVER_LO, "fopen_as_user(): test child[%d] signalled %d.\n",
-		       pid, WTERMSIG(status));	/* XXX what if stopped? */
+		       pid, WTERMSIG(status));
+		errno = EINTR;
+		return NULL;
+	    } else if (WIFSTOPPED(status)) {
+		DEBUG2(DBG_DRIVER_LO, "fopen_as_user(): test child[%d] stopped %d.\n",
+		       pid, WSTOPSIG(status));
 		errno = EINTR;
 		return NULL;
 	    }
@@ -1438,7 +1449,7 @@ open_child(argv, envp, in, out, errfd, flags, uid, gid)
 		/* open /dev/null as the stdin */
 		(void) close(0);
 		if (open("/dev/null", O_RDONLY) < 0) {
-		    /* uggh! failed to open /dev/null, quit */
+		    /* uggh! failed to open /dev/null, quit without flush */
 		    _exit(EX_OSFILE);
 		}
 	    }
@@ -1461,7 +1472,7 @@ open_child(argv, envp, in, out, errfd, flags, uid, gid)
 		/* open /dev/null as stdout */
 		(void) close(1);
 		if (open("/dev/null", O_WRONLY) < 0) {
-		    /* uggh! failed to open /dev/null, quit */
+		    /* uggh! failed to open /dev/null, quit without flush */
 		    _exit(EX_OSFILE);
 		}
 	    }
@@ -1479,7 +1490,7 @@ open_child(argv, envp, in, out, errfd, flags, uid, gid)
 	} else {
 	    (void) close(2);
 	    if (open("/dev/null", O_WRONLY) < 0) {
-		/* uggh! failed to open /dev/null, quit */
+		/* uggh! failed to open /dev/null, quit witout flush */
 		_exit(EX_OSFILE);
 	    }
 	}
