@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/transports:RELEASE-3_2_0_97:smtplib.c,v 1.26 1997/06/01 17:35:13 woods Exp"
+#ident	"@(#)smail/src/transports:RELEASE-3_2_0_98:smtplib.c,v 1.29 1997/09/04 17:49:25 woods Exp"
  */
 
 /*
@@ -82,7 +82,8 @@
 #define REPLY_WILLFORWARD	0x251	/* user address will be forwarded */
 #define REPLY_START_DATA	0x354	/* okay to start sending message */
 #define REPLY_DOWN		0x421	/* remote SMTP closing down */
-#define REPLY_STORAGE_FULL	0x552	/* remote storage full */
+#define REPLY_STORAGE_FULL	0x452	/* remote storage full */
+#define REPLY_UNKNOWN_USER	0x550	/* remote says mailbox unknown */
 /* pseudo-reply codes */
 #define REPLY_PROTO_ERROR	0x498	/* protocol error on read */
 #define REPLY_TIMEOUT		0x499	/* timeout on read, or EOF on read */
@@ -278,6 +279,8 @@ smtp_startup(smtpb, error_p, try_ehlo)
 		    smtpb->smtp_flags |= ESMTP_one;
 		} else if (strncmpic(cp, "XQUE", keywordlength) == 0) {
 		    smtpb->smtp_flags |= ESMTP_queue;
+		} else if (strncmpic(cp, "EXPN", keywordlength) == 0) {
+		    ; /* NOP XXX smail offers this but can ignore it in here */
 		} else {
 		    char sc = *(cp+keywordlength);
 		    DEBUG1(DBG_DRIVER_MID, "destination offers unsupported ESMTP option %s.\n", cp);
@@ -508,8 +511,10 @@ smtp_send(smtpb, addr, succeed, defer, fail, error_p)
 	 * each recipient specified individually.
 	 * Possible responses:
 	 *  250, 251 - okay, or forwarded (continue conversation)
-	 *  451, 452 - remote error    (try again later)
 	 *  421 - connection closing   (try again later)
+	 *  451 - remote error         (try again later)
+	 *  452 - remote storage full  (try this addr again later)
+	 *  550 - unknown user         (return this addr to sender)
 	 *  5xx - failure	       (return message to sender)
 	 */
 	smtp_out.i = 0;
@@ -519,27 +524,38 @@ smtp_send(smtpb, addr, succeed, defer, fail, error_p)
 	reply = write_command_maybe_wait(smtpb, smtpb->long_timeout,
 					 smtp_out.p, (int) smtp_out.i, &reply_text);
 
-	if (reply == REPLY_STORAGE_FULL) {
+	if (REPLY_GROUP(reply) == NEGATIVE_TRY_AGAIN) {
+	    if (reply != REPLY_STORAGE_FULL) {
+		*error_p = try_again(tp, reply_text);
+		insert_addr_list(cur, defer, *error_p);
+		insert_addr_list(okay, defer, *error_p);
+		do_smtp_shutdown(smtpb, reply);
+		return FAIL;
+	    }
 	    *error_p = remote_full(tp, reply_text);
-	    insert_addr_list(cur, defer, *error_p);
-	    next = NULL;
-	} else if (REPLY_GROUP(reply) == NEGATIVE_TRY_AGAIN) {
-	    *error_p = try_again(tp, reply_text);
-	    insert_addr_list(cur, defer, *error_p);
-	    insert_addr_list(okay, defer, *error_p);
-	    do_smtp_shutdown(smtpb, reply);
-	    return FAIL;
+	    cur->succ = *defer;
+	    *defer = cur;
 	} else if (REPLY_GROUP(reply) == NEGATIVE_FAILED) {
+	    if (reply != REPLY_UNKNOWN_USER) {
+		*error_p = remote_bad_address(tp, reply_text);
+		insert_addr_list(cur, fail, *error_p);
+		insert_addr_list(okay, fail, *error_p);
+		do_smtp_shutdown(smtpb, reply);
+		return FAIL;		/* unknown codes are fatal for all recipients */
+	    }
 	    cur->error = remote_bad_address(tp, reply_text);
 	    cur->succ = *fail;
 	    *fail = cur;
-	    return FAIL;
 	} else {
 	    /* successful thus far */
 	    cur->succ = okay;
 	    okay = cur;
 	}
     }
+    if (! okay) {
+	do_smtp_shutdown(smtpb, REPLY_NOT_ACCEPTABLE);
+	return FAIL;
+    }
 
     /*
      * say that we will next be sending the actual data
@@ -554,7 +570,7 @@ smtp_send(smtpb, addr, succeed, defer, fail, error_p)
 
     if (smtpb->smtp_flags & ESMTP_pipelining) {
 
-	if (reply != REPLY_OK) {
+	if (reply != REPLY_OK) {	/* XXX is this not redundant? */
 	    *error_p = try_again(tp, reply_text);
 	    insert_addr_list(addr, defer, *error_p);
 	    do_smtp_shutdown(smtpb, reply);
@@ -562,13 +578,15 @@ smtp_send(smtpb, addr, succeed, defer, fail, error_p)
 	}
 
 	if (flush_command_stream(smtpb, &reply_text) != REPLY_OK) {
+	    *error_p = try_again(tp, reply_text);
+	    insert_addr_list(addr, defer, *error_p);
 	    return FAIL;
 	}
 
 	/* Now read all the responses.  First the MAIL FROM: reply */
 	reply = wait_read_response(smtpb, smtpb->long_timeout,
 				   &reply_text);
-    if (REPLY_GROUP(reply) == NEGATIVE_TRY_AGAIN) {
+	if (REPLY_GROUP(reply) == NEGATIVE_TRY_AGAIN) {
 	    *error_p = try_again(tp, reply_text);
 	    insert_addr_list(addr, defer, *error_p);
 	    do_smtp_shutdown(smtpb, reply);
@@ -583,24 +601,33 @@ smtp_send(smtpb, addr, succeed, defer, fail, error_p)
 	addr = okay;
 	okay = NULL;
 
+	/* ... next the reply from each RCPT TO: command */
 	for (cur = addr; cur; cur = next) {
 	    next = cur->succ;
 
 	    reply = wait_read_response(smtpb, smtpb->long_timeout,
 				       &reply_text);
 
-	    if (reply == REPLY_STORAGE_FULL) {
+	    if (REPLY_GROUP(reply) == NEGATIVE_TRY_AGAIN) {
+		if (reply != REPLY_STORAGE_FULL) {
+		    *error_p = try_again(tp, reply_text);
+		    insert_addr_list(cur, defer, *error_p);
+		    insert_addr_list(okay, defer, *error_p);
+		    do_smtp_shutdown(smtpb, reply);
+		    return FAIL;
+		}
 		*error_p = remote_full(tp, reply_text);
-		insert_addr_list(cur, defer, *error_p);
-		next = NULL;
-	    } else if (REPLY_GROUP(reply) == NEGATIVE_TRY_AGAIN) {
-		*error_p = try_again(tp, reply_text);
-		insert_addr_list(cur, defer, *error_p);
-		insert_addr_list(okay, defer, *error_p);
-		do_smtp_shutdown(smtpb, reply);
-		return FAIL;
+		cur->succ = *defer;
+		*defer = cur;
 	    } else if (REPLY_GROUP(reply) == NEGATIVE_FAILED) {
-		cur->error =  remote_bad_address(tp, reply_text);
+		if (reply != REPLY_UNKNOWN_USER) {
+		    *error_p = remote_bad_address(tp, reply_text);
+		    insert_addr_list(cur, fail, *error_p);
+		    insert_addr_list(okay, fail, *error_p);
+		    do_smtp_shutdown(smtpb, reply);
+		    return FAIL;	/* unknown codes are fatal for all recipients */
+		}
+		cur->error = remote_bad_address(tp, reply_text);
 		cur->succ = *fail;
 		*fail = cur;
 	    } else {
@@ -609,6 +636,7 @@ smtp_send(smtpb, addr, succeed, defer, fail, error_p)
 		okay = cur;
 	    }
 	}
+	/* ... and finally the reply from the DATA command */
 	reply = wait_read_response(smtpb, smtpb->long_timeout,
 				   &reply_text);
     }
