@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/transports:RELEASE-3_2_0_97:tcpsmtp.c,v 1.30 1997/05/23 17:46:33 woods Exp"
+#ident	"@(#)smail/src/transports:RELEASE-3_2_0_98:tcpsmtp.c,v 1.33 1997/08/30 20:57:55 woods Exp"
  */
 
 /*
@@ -17,7 +17,6 @@
 
 #define NEED_SOCKETS
 #include <stdio.h>
-#include <signal.h>
 #include <ctype.h>
 #include <errno.h>
 #include <sys/types.h>
@@ -577,6 +576,14 @@ set_timeout(timeout, attr)
 }
 
 
+/*
+ * import h_errno; many systems don't define it in <netdb.h>
+ */
+
+#if defined(HAVE_BIND) && !defined(OBSOLETE_RESOLVER)
+extern int h_errno;
+#endif
+
 /*
  * tcpsmtp_connect - return a socket connected to the remote host
  *
@@ -615,12 +622,7 @@ tcpsmtp_connect(remote_host, ip_addr, family, service, error)
         port = smtp_service->s_port;
     }
 
-    bzero((char *)&sin, sizeof(sin));
-    sin.sin_addr = *ip_addr;
-    sin.sin_family = family;
-    sin.sin_port = port;
-    s = socket(AF_INET, SOCK_STREAM, 0);
-    if (s < 0) {
+    if ((s = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
         if (save_error) {
             xfree(save_error);
         }
@@ -628,7 +630,41 @@ tcpsmtp_connect(remote_host, ip_addr, family, service, error)
         return -1;
     }
 
-    if (connect(s, (struct sockaddr *)&sin, sizeof(sin)) < 0) {
+    if (listen_name) {
+	struct hostent *hostentp;
+	struct sockaddr_in lsin;	/* local (source) inet socket address */
+
+	if (!(hostentp = gethostbyname(listen_name))) {
+	    if (save_error) {
+		xfree(save_error);
+	    }
+#if defined(HAVE_BIND) && !defined(OBSOLETE_RESOLVER)
+	    *error = save_error = xprintf("gethostbyname(%s): %s", listen_name, hstrerror(h_errno));
+#else
+	    *error = save_error = xprintf("gethostbyname(%s): failed.", listen_name);
+#endif
+	    return -1;
+	}
+	memset((char *) &lsin, '\0', sizeof(lsin));
+	memcpy((char *) &lsin.sin_addr, hostentp->h_addr_list[0], sizeof(struct in_addr));
+	lsin.sin_family = family;
+	lsin.sin_port = 0;		/* any old port will do.... */
+	DEBUG2(DBG_DRIVER_MID, "\nbinding socket to local source addr (%s)[%s]... ", listen_name, inet_ntoa(lsin.sin_addr));
+	if (bind(s, (struct sockaddr *) &lsin, sizeof(lsin)) < 0) {
+	    if (save_error) {
+		xfree(save_error);
+	    }
+	    *error = save_error = xprintf("bind(%s[%s]): %s", listen_name, inet_ntoa(lsin.sin_addr), strerror(errno));
+	    return -1;
+	}
+    }
+
+    memset((char *) &sin, '\0', sizeof(sin));
+    sin.sin_addr = *ip_addr;
+    sin.sin_family = family;
+    sin.sin_port = port;
+
+    if (connect(s, (struct sockaddr *) &sin, sizeof(sin)) < 0) {
         if (save_error) {
             xfree(save_error);
         }
