@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_97:verify.c,v 1.18 1997/08/06 06:17:51 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_98:verify.c,v 1.19 1997/08/26 18:33:21 woods Exp"
  */
 
 /*
@@ -132,7 +132,7 @@ verify_addr_list(in, okay, defer, fail)
 	    }
 	    break;
 
-	default:
+	default:			/* remote forms covered by smtp_remote_allow */
 	    /*
 	     * a remote form may point to the local host, if so
 	     * put it back on the input, otherwise it goes on the
